/***************************************
    This is part of frox: A simple transparent FTP proxy
    Copyright (C) 2000 James Hollingshead

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    common.h -- miscellaneous definitions.
					
***************************************/

#ifndef COMMON_H
#define COMMON_H		/*+ To stop multiple inclusions. + */

#include "config.h"

#include <stdio.h>
#include <sys/types.h>

#if STDC_HEADERS
# include <stdlib.h>
# include <string.h>
#elif HAVE_STRINGS_H
# include <strings.h>
#endif				/* STDC_HEADERS */

#if HAVE_UNISTD_H
# include <unistd.h>
#endif

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>

#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif				/* HAVE_SYS_TIME_H */
#endif				/* TIME_WITH_SYS_TIME */

#include "sstr.h"

/* Defines */
#ifdef DEBUG
#define debug(S) fprintf(stderr,S)
#define debug2(S,T) fprintf(stderr,S,T)
#else
#define debug(S)
#define debug2(S,T)
#endif

#define debug_err(S)  fprintf(stderr, "ERROR: \"%s\" at line %d of %s\n", S, \
                      __LINE__, __FILE__);
#define debug_perr(S) fprintf(stderr, \
                      "ERROR: \"%s: %s\" at line %d of %s\n", S, \
                      errno<sys_nerr ? sys_errlist[errno] : "Unknown error", \
                      __LINE__, __FILE__);

#define TRUE 1
#define FALSE 0

#define DM    242
#define IP    244
#define WILL  251
#define WONT  252
#define DO    253
#define DONT  254
#define IAC   255

#define BUF_LEN 4097
#define MAX_LINE_LEN 180	/* For control connections */

#define GET_CLNT 1
#define GET_SRVR 2
#define GET_BOTH 3

/*Log levels */
#define INFO   20		/* Who connects, etc. */
#define IMPORT 15		/* Startup messages as well. */
#define ERROR  10		/* Non critical errors */
#define ATTACK  5		/* Suspicious stuff */

/* Error codes */
typedef enum _exitreason {
	TIMEOUT,
	CLIENT_CLOSE,
	SERVER_CLOSE,
	NO_CLIENT_DATA,		/* Unable to connect to it */
	NO_SERVER_DATA,
	CLIENT_RUBBISH,		/* Malformed control stream */
	SERVER_RUBBISH,
	MALLOC,
	SOCKET,
	NO_MESSAGE
} exitreason;


typedef enum _socketuse {
	ACTV,
	PASV,
	CTRL
} socketuse;

typedef struct _connection {
	struct sockaddr_in address;
	int fd;

	sstr *buf;
} connection;

typedef struct _session_info {
	connection client_control;
	connection server_control;
	connection client_data;
	connection server_data;
	sstr *server_name;

	enum { ACTIVE, PASSIVE, APCONV } mode;
	enum { UPLOAD, DOWNLOAD, NEITHER } state;
	int listen;		/*fd we are listening on */
} session_info;

#include "configs.h"
#include "misc.h"

/*******************
**Global Variables**
********************/
session_info *info;
extern pid_t cmgrpid, tdatapid;

#endif				/* COMMON_H */
